@@ -501,8 +501,17 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
       }
       break;
     }
-    case GST_MESSAGE_ELEMENT:
-    {
+    case GST_MESSAGE_ELEMENT:{
+      if (gst_is_missing_plugin_message (msg)) {
+        gchar *desc;
+
+        desc = gst_missing_plugin_message_get_description (msg);
+        gst_print ("Missing plugin: %s\n", desc);
+        g_free (desc);
+        play->missing = g_list_prepend (play->missing, gst_message_ref (msg));
+        break;
+      }
+      // Will return GST_NAVIGATION_MESSAGE_INVALID if it's not a navigation message
       GstNavigationMessageType mtype = gst_navigation_message_get_type (msg);
       if (mtype == GST_NAVIGATION_MESSAGE_EVENT) {
         GstEvent *ev = NULL;
@@ -684,14 +693,6 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
       break;
     }
     default:
-      if (gst_is_missing_plugin_message (msg)) {
-        gchar *desc;
-
-        desc = gst_missing_plugin_message_get_description (msg);
-        gst_print ("Missing plugin: %s\n", desc);
-        g_free (desc);
-        play->missing = g_list_append (play->missing, gst_message_ref (msg));
-      }
       break;
   }
 
@@ -1289,6 +1290,7 @@ play_cycle_track_selection (GstPlay * play, GstPlayTrackType track_type,
       }
       break;
     default:
+      g_mutex_unlock (&play->selection_lock);
       return;
   }
 
@@ -1507,11 +1509,6 @@ keyboard_cb (const gchar * key_input, gpointer user_data)
     case 't':
       play_switch_trick_mode (play);
       break;
-    case 27:                   /* ESC */
-      if (key_input[1] == '\0') {
-        g_main_loop_quit (play->loop);
-        break;
-      }
     case 'a':
     case 'A':
       play_cycle_track_selection (play, GST_PLAY_TRACK_TYPE_AUDIO, key == 'a');
@@ -1531,6 +1528,12 @@ keyboard_cb (const gchar * key_input, gpointer user_data)
     case 'm':
       play_toggle_audio_mute (play);
       break;
+    case 27:                   /* ESC */
+      if (key_input[1] == '\0') {
+        g_main_loop_quit (play->loop);
+        break;
+      }
+      /* FALLTHROUGH */
     default:
       if (strcmp (key_input, GST_PLAY_KB_ARROW_RIGHT) == 0) {
         relative_seek (play, +0.08);
